---------------------------------------------
-- module for UART interface communication --
-- variable Bd, 1 start, 8 data, 1 stop   --
---------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity UARTInterface is
	generic 	(	Frequency				:  natural	:= 100;							-- frequency of the module clock input in MHz
					Baud						:  natural  := 9600);						-- baudrate
	port 		(	-- system parameter --
					Clock	  					:  in std_logic;								-- module clock input
					Reset	  					:  in std_logic;								-- reset, active low
					-- UART signals --
					RxD						:  in std_logic;								-- serial receivce
					TxD						: out std_logic;								-- serial transmit
					-- AXI4 Stream transmit interface --
					Tx_Valid					:  in std_logic;								
					Tx_Data					:  in std_logic_vector(7 downto 0);		
					Tx_Ack					: out std_logic;													
					-- AXI4 Stream receive interface --
					Rx_Valid					: out std_logic;								
					Rx_Data					: out std_logic_vector(7 downto 0);		
					Rx_Ack					:  in std_logic);								
end UARTInterface;

architecture arch of UARTInterface is
-- constants --
constant DividerFactor : natural := Frequency * 1000000 / Baud;	
-- signals --
type RxStates is (Idle, Sync, Data);										
type TxStates is (Idle, Start, Data, Back);								
signal RxState									: RxStates;											
signal TxState									: TxStates;											
signal RxD_Int, RxD_Int_Old, Baudtick	: std_logic;																				
signal RxD_SR									: std_logic_vector(8 downto 0);				
signal TxD_SR									: std_logic_vector(7 downto 0);				
signal RxCounter 								: natural range 0 to 9;							
signal TxCounter 								: natural range 0 to 8;							
signal BaudCounter 							: natural range 0 to DividerFactor-1;											
begin


	-----------------------
	-- generate baudtick --
	-----------------------
	
	BaudTickGenerator: process(Clock, Reset)
	begin
		if Reset='0' then BaudCounter <= 0; 
								Baudtick		<= '0';
		
		elsif rising_edge(Clock) then 
			-- default --
			Baudtick	<= '0';
			-- increment counter --
			if BaudCounter<DividerFactor-1 then BaudCounter <= BaudCounter + 1; else BaudCounter <= 0; end if;
			-- generate baudtick
			if BaudCounter=0 then Baudtick <= '1'; end if;
			-- set the baudtick counter to the middle of the incoming bit
			if RxState=Sync then BaudCounter <= DividerFactor/2; Baudtick <= '0'; end if;
		end if;
	end process BaudTickGenerator;

	-------------
	-- Receive --
	-------------

	Receive: process(Clock, Reset)
	begin
		if Reset='0' then RxState 		<= Idle;
								RxCounter 	<= 0;
								Rx_Data		<= (others => '0');
								Rx_Valid		<= '0';
								RxD_SR 		<= (others => '0');
		
		elsif rising_edge(Clock) then
			-- default --
			if Rx_Ack='1' then Rx_Valid <= '0'; end if;
			-- clock in the incoming data signals --
			RxD_Int 		<= RxD;
			RxD_Int_Old <= RxD_Int;
			-- State Maschine --
			case RxState is
				when Idle	=> -- check for clock edge --
									if RxD_Int_Old='1' and RxD_Int='0' then RxCounter <= 0; RxState <= Sync; end if;
				when Sync	=> -- take this state to set the baudtick to the middle of the incomming bit --
									RxState <= Data;
				when Data	=> -- check for baudtick --
									if Baudtick='1' then
										-- 9 bits received?
										if RxCounter=9 	then 
											-- check start and stop bit - when ok - set data to module ports
											if RxD_SR(0)='0' and RxD_Int='1' then Rx_Data <= RXD_SR(8 downto 1); Rx_Valid <= '1'; end if;
											-- new state --
											RxState <= Idle;
										else 
											-- increment counter --
											RxCounter <= RxCounter + 1;
										end if;
									end if;
				when others	=> -- default --
									RxState <= Idle;
			end case;
			-- save incoming data in a shift register --
			if Baudtick='1' then RxD_SR <= RxD_Int & RxD_SR(8 downto 1); end if;
		end if;
	end process Receive;

	--------------
	-- Transmit --
	--------------

	Transmit: process(Clock, Reset)
	begin
		if Reset='0' then TxState 		<= Idle;
								TxCounter 	<= 0;
								TxD_SR		<= (others => '0');
								Tx_Ack		<= '0';
								TxD			<= '1';
		
		elsif rising_edge(Clock) then
			-- default
			Tx_Ack <= '0';
			-- State Maschine --
			case TxState is
				when Idle	=> -- default --
									TxD <= '1';
									-- check for new data --
									if Tx_Valid='1' then TxCounter <= 0; TxD_SR <= Tx_Data; Tx_Ack <= '1'; TxState <= Start; end if;
				when Start	=> -- check for baudtick - set start bit --
									if Baudtick='1' then TxD <= '0'; TxState <= Data; end if;
				when Data	=> -- check for baudtick --
									if Baudtick='1' then
										-- 8 bit transmitted? --
										if TxCounter=8 then 
											-- set stop bit --
											TxD <= '1';
											-- new state --
											TxState <= Back;
										else 
											-- set data bit - LSB first --
											TxD <= TxD_SR(TxCounter);
											-- increment counter --
											TxCounter <= TxCounter + 1;
										end if;
									end if;
				when Back	=> -- done - back to idle --
									if Baudtick='1' then TxState <= Idle; end if;
				when others	=> -- default --
									TxState <= Idle;
			end case;
		end if;
	end process Transmit;

end arch;