---------------------------------------------------
-- BitRecords Deltaboard UART Demo for User FPGA --
---------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
library UNISIM;
use UNISIM.VComponents.all;

entity User_FPGA is
	port	(	------------------------
				-- differential clock --
				------------------------				
				ExtTakt_P			:    in std_logic;								
				ExtTakt_N			:    in std_logic;												
				---------------------------------------------------
				-- AXI4 Stream Connection to Parameter Interface --
				---------------------------------------------------				
				Ctrl_Rx_Clk			:    in std_logic;								
				Ctrl_Rx_Valid		:    in std_logic;								
				Ctrl_Rx_Data		:    in std_logic_vector(7 downto 0);		
				Ctrl_Rx_Ack			:   out std_logic;								
				Ctrl_Tx_Clk			:   out std_logic;								
				Ctrl_Tx_Valid		:   out std_logic;								
				Ctrl_Tx_Data		:   out std_logic_vector(7 downto 0);		
				Ctrl_Tx_Ack			:    in std_logic;								
				-------------------------------------------------
				-- AXI4 Stream Data Connection to IO Interface --
				-------------------------------------------------
				IO_IF_Rx_Clk		:    in std_logic;								
				IO_IF_Rx_Valid		:    in std_logic;								
				IO_IF_Rx_Data		:    in std_logic_vector(15 downto 0);		
				IO_IF_Rx_Ack		:   out std_logic;								
				IO_IF_Tx_Clk		:   out std_logic;								
				IO_IF_Tx_Valid		:   out std_logic;								
				IO_IF_Tx_Data		:   out std_logic_vector(15 downto 0);		
				IO_IF_Tx_Ack		:    in std_logic;								
				-------------------------------------------------------
				-- AXI4 Stream Data Connection to FMC Connector FPGA --
				-------------------------------------------------------
				Con_IF_Rx_Clk		:    in std_logic;								
				Con_IF_Rx_Valid	:    in std_logic;								
				Con_IF_Rx_Data		:    in std_logic_vector(15 downto 0);		
				Con_IF_Rx_Ack		:   out std_logic;								
				Con_IF_Tx_Clk		:   out std_logic;								
				Con_IF_Tx_Valid	:   out std_logic;								
				Con_IF_Tx_Data		:   out std_logic_vector(15 downto 0);		
				Con_IF_Tx_Ack		:    in std_logic;								
				--------------------------
				-- UART Debug Interface --
				--------------------------
				UART_RxD				:    in std_logic;								
				UART_TxD				:   out std_logic;												
				-----------------
				-- DDR2 SDRAM  --
				-----------------			
				DDR2_A				:   out std_logic_vector(13 downto 0);
				DDR2_DQ				: inout std_logic_vector(7 downto 0);
				DDR2_BA				:   out std_logic_vector(2 downto 0);
				DDR2_DQS_P			: inout std_logic;
				DDR2_DQS_N			: inout std_logic;
				DDR2_DM				:   out std_logic;
				DDR2_RAS_N			:   out std_logic;
				DDR2_CAS_N			:   out std_logic;
				DDR2_ODT				:   out std_logic;
				DDR2_CKE				:   out std_logic;
				DDR2_CLK_P			:   out std_logic;
				DDR2_CLK_N			:   out std_logic;
				DDR2_WE_N			:   out std_logic;
				---------------
				-- SPI Flash --
				---------------
				Flash_MISO			:  in std_logic;
				Flash_MOSI			: out std_logic;
				Flash_nCS			: out std_logic;
				Flash_Clk			: out std_logic;
				-------------
				-- SD Card --
				-------------
				SD_Dat				: inout std_logic_vector(3 downto 0);
				SD_Cmd				:   out std_logic;
				SD_Clk				:   out std_logic;
				---------------------
				-- Switch and LEDs --
				---------------------
--				Switch				:    in std_logic;								-- XC6SLX150 Version only
				USERLED201			:   out std_logic);
--				USERLED202			:   out std_logic;								-- XC6SLX150 Version only
--				USERLED203			:   out std_logic);								-- XC6SLX150 Version only
end User_FPGA;

architecture Behavioral of User_FPGA is
signal Clock_100_Internal, Clock_FB, Clock_50_Pll, Clock_50, Locked, Reset_ALow,
		 UART_Rx_Valid, UART_Rx_Ack, UART_Tx_Valid, UART_Tx_Ack, Rx_Valid,
		 Rx_Ack, Tx_Valid, Tx_Ack																: std_logic;
signal UART_Rx_Data, UART_Tx_Data, Rx_Data, Tx_Data									: std_logic_vector(7 downto 0);
begin

	--------------------------------
	-- Clock and Reset Generation --
	--------------------------------
	
	-- input buffer for the differential 100 MHz signal
	CLKBUFIN: IBUFGDS
	port map		(	O  						=> Clock_100_Internal,				-- Clock buffer output
						I  						=> ExtTakt_P,							-- Diff_p clock buffer input 
						IB 						=> ExtTakt_N);							-- Diff_n clock buffer input

	-- generate a 50 MHz clock source
	SystemClock: PLL_BASE
	generic map	(	BANDWIDTH            => "OPTIMIZED",
						CLK_FEEDBACK         => "CLKFBOUT",
						COMPENSATION         => "SYSTEM_SYNCHRONOUS",
						DIVCLK_DIVIDE        => 1,
						CLKFBOUT_MULT        => 10,
						CLKFBOUT_PHASE       => 0.000,
						CLKOUT0_DIVIDE       => 20,
						CLKOUT0_PHASE        => 0.000,
						CLKOUT0_DUTY_CYCLE   => 0.500,
						CLKIN_PERIOD         => 10.0,
						REF_JITTER           => 0.010)
	port map		(	CLKFBOUT            	=> Clock_FB,
						CLKOUT0             	=> Clock_50_Pll,   	
						CLKOUT1             	=> open,	
						CLKOUT2             	=> open,		
						CLKOUT3             	=> open,	
						CLKOUT4             	=> open,
						CLKOUT5             	=> open,
						LOCKED              	=> Locked,
						RST                 	=> '0',
						CLKFBIN             	=> Clock_FB,
						CLKIN               	=> Clock_100_Internal);
	
	-- global output buffer
	T50Buf : BUFG
	port map		(	O   						=> Clock_50,
						I   						=> Clock_50_Pll);

	-- set the internal active low reset signal
	Reset_ALow <= Locked;
	
	
	--------------------
	-- UART Interface --
	--------------------
	
	UART: entity Work.UARTInterface
	generic map (	Frequency				=> 50,									-- frequency of the module clock input in MHz
						Baud						=> 115200)								-- baudrate
	port map 	(	-- system parameter --
						Clock	  					=> Clock_50,							-- module clock input
						Reset	  					=> Reset_ALow,							-- reset, active low
						-- UART signals --
						RxD						=> UART_RxD,							-- serial receivce
						TxD						=> UART_TxD,							-- serial transmit
						-- AXI4 Stream transmit interface --
						Tx_Valid					=> UART_Tx_Valid,								
						Tx_Data					=> UART_Tx_Data,		
						Tx_Ack					=> UART_Tx_Ack,													
						-- AXI4 Stream receive interface --
						Rx_Valid					=> UART_Rx_Valid,							
						Rx_Data					=> UART_Rx_Data,	
						Rx_Ack					=> UART_Rx_Ack);


	-----------------------
	-- signal processing --
	-----------------------
	
	-- we use two small fifos for buffering the incoming and outgoing data --
	RxFIFO: entity Work.AXI4_FIFO_128x8
	port map		(	-- clock and reset --
						s_aclk					=> Clock_50,
						s_aresetn				=> Reset_ALow,
						-- AXI4 Stream receive interface = write data to fifo --
						s_axis_tvalid			=> UART_Rx_Valid,
						s_axis_tdata			=> UART_Rx_Data,
						s_axis_tready			=> UART_Rx_Ack,
						-- interface to signal processing process = read data from fifo --
						m_axis_tvalid			=> Rx_Valid,
						m_axis_tdata			=> Rx_Data,
						m_axis_tready			=> Rx_Ack);

	TxFIFO: entity Work.AXI4_FIFO_128x8
	port map		(	-- clock and reset --
						s_aclk					=> Clock_50,
						s_aresetn				=> Reset_ALow,
						-- interface to signal processing process = write data to fifo --
						s_axis_tvalid			=> Tx_Valid,
						s_axis_tdata			=> Tx_Data,
						s_axis_tready			=> Tx_Ack,
						-- AXI4 Stream transit interface = read data from fifo --
						m_axis_tvalid			=> UART_Tx_Valid,
						m_axis_tdata			=> UART_Tx_Data,
						m_axis_tready			=> UART_Tx_Ack);
						
	-- change all incoming data byte to big ascii letter and send it back --
	SetBigLetter: process(Clock_50, Reset_ALow)
	begin
		if Reset_ALow='0' then	Tx_Valid <= '0';
										Tx_Data	<= (others => '0');
										
		elsif rising_edge(Clock_50) then
			-- set transmit valid flag back to low --
			if Tx_Ack='1' then Tx_Valid <= '0'; end if;
			-- check for new data byte - if the outgoing fifo is not full start the processing --
			if Rx_Valid='1' and Tx_Ack='1' then
				-- check for a valid letter --
				if (Rx_Data>x"40" and Rx_Data<x"5B") or (Rx_Data>x"60" and Rx_Data<x"7B") then
					-- set bit 5 low and send it back --
					Tx_Valid <= '1';
					Tx_Data	<= Rx_Data and x"5F";
				end if;
			end if;
		end if;
	end process SetBigLetter;
		
	-- set the fifo ready flag asynchronous when take out the data for processing --
	Rx_Ack <= '1' when Rx_Valid='1' and Tx_Ack='1' else '0';
	

	--------------------------------------
	-- ports not used for the UART demo --
	--------------------------------------
	
	-- AXI4 Stream Connection to Parameter Interface --	
	Ctrl_Rx_Ack			<= '0';								
	Ctrl_Tx_Clk			<= '0';								
	Ctrl_Tx_Valid		<= '0';								
	Ctrl_Tx_Data		<= (others => '0');									
	-- AXI4 Stream Data Connection to IO Interface --	
	IO_IF_Rx_Ack		<= '0';							
	IO_IF_Tx_Clk		<= '0';								
	IO_IF_Tx_Valid		<= '0';								
	IO_IF_Tx_Data		<= (others => '0');		
	-- AXI4 Stream Data Connection to FMC Connector FPGA --	
	Con_IF_Rx_Ack		<= '0';								
	Con_IF_Tx_Clk		<= '0';								
	Con_IF_Tx_Valid	<= '0';							
	Con_IF_Tx_Data		<= (others => '0');
	-- DDR2 SDRAM  --	
	DDR2_A				<= (others => '0');
	DDR2_DQ				<= (others => 'Z');
	DDR2_BA				<= (others => '0');
	DDR2_DQS_P			<= 'Z';
	DDR2_DQS_N			<= 'Z';
	DDR2_DM				<= '0';
	DDR2_RAS_N			<= '1';
	DDR2_CAS_N			<= '1';
	DDR2_ODT				<= '0';
	DDR2_CKE				<= '0';
	DDR2_CLK_P			<= '0';
	DDR2_CLK_N			<= '0';
	DDR2_WE_N			<= '1';
	-- SPI Flash --
	Flash_MOSI			<= '0';
	Flash_nCS			<= '1';
	Flash_Clk			<= '0';
	-- SD Card --
	SD_Dat				<= (others => 'Z');
	SD_Cmd				<= '0';
	SD_Clk				<= '0';
	-- Switch and LEDs --
	USERLED201			<= '0';	


end Behavioral;